<?php

function papilia_campaign_form_validate($node, &$form) {
    
  $val = (object)$form['values'];
    
  if (empty($val->dfdbid)) {
    form_set_error('dfdbid', t('Donation Form Identifier may not be left empty')); 
  }
  
  $allowed_currencies = $val->currency_selection['currency_selection_enabled'];  
    
  $amount_choices = FALSE;
  if (is_array($val->amt_choice)) {
    $idx = 0;
    foreach ($val->amt_choice as $amt) {
      if (!in_array($amt['chtype'], $allowed_currencies) && $amt['chvalue'] > 0 ) {
        $element_id = 'amt_choice]['.$idx.'][chtype';
        form_set_error($element_id, t('All currencies for hand-picked amounts must be from the list of Available Currencies.'));        
      }
      if (!empty($amt['chvalue'])) {
        $amount_choices = TRUE; //at least one legit choice
      }
      $idx++;
    }
  }
  
  if (!$amount_choices && !$val->free_entry_amount) {
    form_set_error('amt_choice', t('No valid amount choices and free-form entry of the amount is disabled! Donors have no way to indicate amount of money!'));
  }
  
  if (!in_array($val->default_currency, $allowed_currencies)) {
    form_set_error('default_currency', t('Default currency must be one of the Available Currencies!'));
  }

  if (empty($allowed_currencies)) {
    form_set_error('currency_selection', t('The list of Available Currencies may not be empty! Please select at least one currency.'));
  }


  $allowed_funds = $val->funds_selection['funds_selection_enabled'];  
  if (!in_array($val->default_fund, $allowed_funds) && $val->default_fund != -1) {
    form_set_error('default_fund', t('Default fund must be one of the Selected Funds!'));
  }
  
  // Why not?
  //if (empty($allowed_funds)) {
  //  form_set_error('fund_selection', t('The list of Selected Funds may not be empty! Please select at least one fund.'));
  //}

  
}

function papilia_campaign_form(&$node, &$form) {

  require_once (dirname(__FILE__) . '/form.fund.inc');
  $funds_options = papilia_get_funds(TRUE);

  require_once (dirname(__FILE__) . '/form.currency.inc');
  $currencies_options = papilia_get_currencies(TRUE);

  $form['body_field'] = node_body_field($node, t('Introduction'), 0);
  $form['body_field']['body']['#rows'] = 5;
  unset($form['body_field']['#after_build']);
  unset($form['body_field']['teaser_js']);
  unset($form['body_field']['teaser_include']);    

  $form['dfdbid'] = array(
    '#title' => t('Donation Form Identifier'),
    '#type' => 'textfield',
    '#default_value' => $node->dfdbid,
  );

  $form['uri'] = array(
    '#title' => t('Donation Form URI Path'),
    '#type' => 'textfield',
    '#default_value' => $node->uri,
    '#description' => t('You can expose donation form at a URL. The actual URI path will be formed by prepanding URL Prefix indicated in the Papilia Settings'),
  );
  
   $form['thankyou_template'] = array(
    '#title' => t('HTML Template for Donation Payment\'s "Thank you" page'),
    '#type' => 'textarea',
    '#size' => 7,
    '#default_value' => $node->thankyou_template,
    '#description' => 'Note: you can use pre-defined variables: a) %confirmation to display the transaction number of the payment and
                       b) %campaign to display the title of the campaign that donation was made for.',
  );
  
   $form['currency_selection'] = array(
    '#title' => t('Available Currencies'),
    '#description' => t('You can custom-order enabled currencies. Disabled currencies are always alphabetically ordered.'),
    '#type' => 'jquery_selector',
    '#options' => array(
      '#enabled'  =>  $node->currency_selection['currency_selection_enabled'],
      '#disabled' =>  $node->currency_selection['currency_selection_disabled'],
    ),
    '#default_value' => papilia_get_currencies(TRUE),
  );

  $form['default_currency'] = array(
    '#title' => t('Default Currency'),
    '#description' => t('Please make sure the currency is enabled first'),
    '#type' => 'select',
    '#options' => $currencies_options,
    '#default_value' => array($node->default_cid),    
  );    

  
  //---- Amounts AHAH Selection
  
    if (isset($form_state['choice_count'])) {
      $choice_count = $form_state['choice_count'];
    }
    else {
      $choice_count = max(1, empty($node->amt_choice) ? 1 : count($node->amt_choice));
    }
  
    $form_state['choice_count'] = $choice_count++;
  
    // Add a wrapper for the choices and more button.
    $form['choice_wrapper'] = array(
      '#tree' => FALSE,
      '#prefix' => '      
        <label><b>' . t('Available Amounts') . ':</b></label>
        <div class="clear-block" id="amount-choice-wrapper">',
        
      '#suffix' => '</div>
      <div class="form-item"><div class="description">' . 
      t('HINT: If you need to delete an amount option, just set its amount value to 0, please.') .
      '</div></div>',
    );
  
    // Container for just the amount choices.
    $form['choice_wrapper']['amt_choice'] = array(
      '#prefix' => '<div id="amount-choices">',
      '#suffix' => '</div>',
    );

    // Container for just the amount choices.
    $form['choice_wrapper']['choice'] = array(
      '#prefix' => '<div id="amount-choices">',
      '#suffix' => '</div>',
    );
  
    // Add the current choices to the form.
    for ($delta = 0; $delta < $choice_count; $delta++) {
      $amt = isset($node->amt_choice[$delta]['chvalue']) ? $node->amt_choice[$delta]['chvalue'] : 0;
      $type = isset($node->amt_choice[$delta]['chtype']) ? $node->amt_choice[$delta]['chtype'] : 'USD';
  
      $form['choice_wrapper']['amt_choice'][$delta] = _papilia_amount_choice_form($delta, $amt, $type);
    }

    $form['choice_wrapper']['free_entry_amount'] = array(
      '#type' => 'checkbox',
      '#title' => t('Allow free-form entry of the donation amount'),
      '#default_value' => empty($node->free_entry_amount) ? 0 : $node->free_entry_amount,
    );
    
    $form['choice_wrapper']['amount_more'] = array(
      '#type' => 'submit',
      '#value' => t('More choices'),
      '#description' => t("Add more amount choices."),
      '#submit' => array('pap_amount_more_choices_submit'), // If no javascript action.
      '#ahah' => array(
        'path' => 'papilia/amounts/js',
        'wrapper' => 'amount-choices',
        'method' => 'replace',
        'effect' => 'fade',
      ),
    );

  //---- End Amounts AHAH Selection
    
  $form['recurr_periods'] = array(
    '#title' => t('Available Periods for Recurring Donations'),
    '#description' => t('You can custom-order enabled periods. Disabled periods are always alphabetically ordered.'),
    '#type' => 'jquery_selector',
    '#options' => array(
      '#enabled'  =>  $node->recurr_periods['recurr_periods_enabled'],
      '#disabled' =>  $node->recurr_periods['recurr_periods_disabled'],
    ),
    '#default_value' => papilia_get_recurr_periods(),
  );    
    
  $form['donor_fields'] = array(
    '#title' => t('Optional Donor Fields'),
    '#description' => t('These fields are optional. You need to select them if you want 
        these feilds to be collected as part of the donation form, for the current campaign'),
    '#type' => 'select',
    '#options' => papilia_donor_fields(),
    '#default_value' => $node->donor_fields,
    '#multiple' => TRUE,
  );

  $form['funds_selection'] = array(
    '#title' => t('Select Funds'),
    '#description' => t('You can order enabled funds. Disabled funds are always alphabetically ordered.'),
    '#type' => 'jquery_selector',
    '#options' => array(
      '#enabled'  =>  $node->funds_selection['funds_selection_enabled'],
      '#disabled' =>  $node->funds_selection['funds_selection_disabled'],
    ),
    '#default_value' => papilia_get_funds(TRUE),
  );
  
  $funds_options_and_default = array('-1' => PAPILIA_SELECTION_NONE) + $funds_options;
  $form['default_fund'] = array(
    '#title' => t('Default Fund'),
    '#description' => t('Please make sure the fund is enabled first'),    
    '#type' => 'select',
    '#options' => $funds_options_and_default,
    '#default_value' => array($node->default_fid),
  );  

  return $form;
}

function pap_amount_more_choices_submit($form, &$form_state) {  
  // Set the form to rebuild and run submit handlers.
  node_form_submit_build_node($form, $form_state);

  // Make the changes we want to the form state.
  if ($form_state['values']['amount_more']) {
    $n = $_GET['q'] == 'papilia/amounts/js' ? 1 : 5;
    $form_state['choice_count'] = count($form_state['values']['amt_choice']) + $n;
  }
}

function _papilia_amount_choice_form($delta, $value = '', $currency_type = 'USD') {

  $form = array(
    '#tree' => TRUE,
  );

  $form['amt_choice_'.$delta.'_prefix'] = array(
    '#value' => '<div class="pap_amt_choice clear clearfix">',  
  );
  
  // We'll manually set the #parents property of these fields so that
  // their values appear in the $form_state['values']['choice'] array.
  $form['chvalue'] = array(
    '#type' => 'textfield',
    '#title' => t('Amount @n', array('@n' => ($delta + 1))),
    '#default_value' => $value,
    '#parents' => array('amt_choice', $delta, 'chvalue'),
  );

  $form['chtype'] = array(
    '#type' => 'select',
    '#default_value' => $currency_type,
    '#options' => papilia_get_currencies(TRUE),
    '#parents' => array('amt_choice', $delta, 'chtype'),
  );

  $form['amt_choice_'.$delta.'_suffix'] = array(
    '#value' => '</div><div class="clear"></div>',  
  );
    
  return $form;
}

function papilia_campaign_list() {

  $ret_url = array('query' => 'destination=admin/settings/papilia/list');

  $campaigns = papilia_get_campaigns(); //Defined in .module file
   
  $out  = '<div ="papilia_campaign_management">';
  
  $out .= '<div ="papilia_campaign_listing">';  

  if (sizeof($campaigns)>0) {
    $out .= "
      <table class=\"papilia_campaign_list\">
      <tr>
        <th><b>". t("Title") ."</b></th>
        <th><b>". t("Status") ."</b></th>        
        <th colspan=\"2\" align=\"center\"><b>". t("Control") ."</b></th>                      
      </tr>
    ";
  
    foreach ( $campaigns as $obj ) {
      $out .="<tr>";
        if (empty($obj->uri)) {
          $campaign_title = '<p>' . $obj->title . '</p>';
        }
        else {
          $url = get_papilia_url_prefix() ."/". $obj->uri;
          $campaign_title = l($obj->title, $url);
        }
      $out .= "<td class=\"pctitle\">". $campaign_title ."</td>";
      $enabled = ( $obj->status ) ? t("Published") : t("Unpublished");
      $out .= "<td>". $enabled ."</td>";       
      $out .= "<td>". l(t('Edit'), 'node/'. $obj->nid .'/edit', $ret_url) ."</td>"; 
      $out .= "<td>". l(t('Delete'), 'node/'. $obj->nid .'/delete', $ret_url) ."</td>";         
      $out .= "</tr>";    
    }
    
    $out .= "</table>";
  }
  else { // no campaigns, yet
    $out .= t("Campaigns list is empty, at this point.");
  }
  $out .= '</div>';
    
  $out .= '[ ' . l(t('Add Campaign'), 'node/add/papilia-campaign', $ret_url) . ' ]';
  $out .= '</div>';
  
  return $out;

}

function papilia_campaign_amounts_js() {
  $n_mpath = drupal_get_path('module', 'node');
  include_once ($n_mpath . '/node.pages.inc');
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  // Get the form from the cache.
  $form = form_get_cache($form_build_id, $form_state);
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  // We will run some of the submit handlers so we need to disable redirecting.
  $form['#redirect'] = FALSE;
  // We need to process the form, prepare for that by setting a few internals
  // variables.
  $form['#post'] = $_POST;
  $form['#programmed'] = FALSE;
  $form_state['post'] = $_POST;
  // Build, validate and if possible, submit the form.
  drupal_process_form($form_id, $form, $form_state);
  // This call recreates the form relying solely on the form_state that the
  // drupal_process_form set up.
  $form = drupal_rebuild_form($form_id, $form_state, $args, $form_build_id);
  // Render the new output.
  $choice_form = $form['choice_wrapper']['amt_choice'];
  unset($choice_form['#prefix'], $choice_form['#suffix']); // Prevent duplicate wrappers.
  $output = theme('status_messages') . drupal_render($choice_form);

  drupal_json(array('status' => TRUE, 'data' => $output));
}